/*---------------------------------------------------------------------------*/
/*                      INGENIOUS TECHNOLOGY CO., LTD.                       */
/*                                                                           */
/*                   MOUSE.C    For Mouse Function Call Utility              */
/*                   Support For C                                           */
/*                                                                           */
/*      Written on 10/01/92 by Dannes Hsieh                                  */
/*---------------------------------------------------------------------------*/

#include <dos.h>
#include "mouse.h"

/*  Function 0.    Mouse Reset     */
char Reset()
{
  union REGS ireg, oreg;

  ireg.x.ax = 0;
  int86(0x33, &ireg, &oreg);

  if (oreg.x.ax == 0)
     return (0);
  else
     return (1);
}



/*  Function 0.     Get mouse button number    */
char Get_Button_Num()
{
  union REGS ireg, oreg;

  ireg.x.ax = 0;
  int86(0x33, &ireg, &oreg);

  if (oreg.x.bx == 3)
     return (3);
  if (oreg.x.bx == 2)
     return (2);
  return 0;
}



/*  Function 1.    Show mouse cursor   */
void Show_Mouse()
{
  union REGS ireg;

  ireg.x.ax = 1;
  int86(0x33, &ireg, &ireg);
}



/* Function 2.     Hide mouse cursor   */
void Hide_Mouse()
{
  union REGS ireg;

  ireg.x.ax = 2;
  int86(0x33, &ireg, &ireg);
}



/* Function 3. Check if left button is pressed.   */
char Left_Pressed()
{
  union REGS ireg, oreg;

  ireg.x.ax = 3;
  int86(0x33, &ireg, &oreg);
  return (oreg.x.bx & 1);
}



/* Function 3. Check if right button is pressed.   */
char Right_Pressed()
{
  union REGS ireg, oreg;

  ireg.x.ax = 3;
  int86(0x33, &ireg, &oreg);
  return (oreg.x.bx & 2);
}



/* Function 3. Check if middle button is pressed.   */
char Mid_Pressed()
{
  union REGS ireg, oreg;

  ireg.x.ax = 3;
  int86(0x33, &ireg, &oreg);
  return (oreg.x.bx & 4);
}



/* Function 3. Check if the given button is pressed.  */
char Button_Pressed(char Button)
{
  union REGS ireg, oreg;

  ireg.x.ax = 3;
  int86(0x33, &ireg, &oreg);
  return(oreg.x.bx == Button);
}



/* Function 3.   Get position of cursor   */
void Get_XY(int *x, int *y)
{
  union REGS ireg, oreg;

  ireg.x.ax = 3;
  int86(0x33, &ireg, &oreg);

  *x = oreg.x.cx;
  *y = oreg.x.dx;
}



/* Function 3.   Get position of cursor in text  */
void Get_T_XY(int *Tx, int *Ty)
{
  int x, y;

  Get_XY(&x, &y);
  *Tx = x/8 + 1;
  *Ty = y/8 + 1;
}



/* Function 4.   Set position of cursor  */
void Set_XY(int x, int y)
{
  union REGS ireg;

  ireg.x.ax = 4;
  ireg.x.cx = x;
  ireg.x.dx = y;
  int86(0x33, &ireg, &ireg);
}



/* Function 4.   Set position of cursor in text  */
void Set_T_XY(int Tx, int Ty)
{
  int x, y;

  x = (Tx-1) * 8;
  y = (Ty-1) * 8;
  Set_XY(x, y);
}



/* Function 5.   Get button press information  */
int Pressed_Status(int button, int *counter,
                   int *x, int *y)
{
  union REGS ireg, oreg;

  ireg.x.ax = 5;
  ireg.x.bx = button;
  int86(0x33, &ireg, &oreg);

  *counter = oreg.x.bx;
  *x = oreg.x.cx;
  *y = oreg.x.dx;
  return (oreg.x.ax);
}



/* Function 5.   Get which button(s) is(are) pressed  */
int Which_Pressed()
{
  union REGS ireg, oreg;

  ireg.x.ax = 5;
  int86(0x33, &ireg, &oreg);

  return (oreg.x.ax);
}



/* Function 6.   Get button release information  */
int Released_Status(int button, int *counter,
                    int *x, int *y)
{
  union REGS ireg, oreg;

  ireg.x.ax = 6;
  ireg.x.bx = button;
  int86(0x33, &ireg, &oreg);

  *counter = oreg.x.bx;
  *x = oreg.x.cx;
  *y = oreg.x.dx;
  return (oreg.x.ax);
}



/* Function 7.  Set the range of horizontal
                cursor position               */
void Set_X_Range(int min, int max)
{
   union REGS ireg;

   ireg.x.ax = 7;
   ireg.x.cx = min;
   ireg.x.dx = max;
   int86(0x33, &ireg, &ireg);
}



/* Function 8.  Set the range of vertical
                cursor position               */
void Set_Y_Range(int min, int max)
{
   union REGS ireg;

   ireg.x.ax = 8;
   ireg.x.cx = min;
   ireg.x.dx = max;
   int86(0x33, &ireg, &ireg);
}



/* Function 9.  Set graphic cursor block  */
void Set_Graphic_Cursor(int x, int y,
                        unsigned int far *pattern)
{
   union REGS ireg;
   struct SREGS isreg;

   ireg.x.ax = 9;
   ireg.x.bx = x;
   ireg.x.cx = y;
   ireg.x.dx = FP_OFF(pattern);
   isreg.es  = FP_SEG(pattern);
   int86x(0x33, &ireg, &ireg, &isreg);
}



/* Function 10.  Set text cursor   */
void Set_Text_Cursor(int type, int screen_mask,
                               int cursor_mask)
{
   union REGS ireg;

   ireg.x.ax = 10;
   ireg.x.bx = type;
   ireg.x.cx = screen_mask;
   ireg.x.dx = cursor_mask;
   int86(0x33, &ireg, &ireg);
}



/* Function 11. Read mouse motion counters  */
int Motion(int *x, int *y)
{
   union REGS ireg, oreg;

   ireg.x.ax = 11;
   int86(0x33, &ireg, &oreg);

   *x = oreg.x.cx;
   *y = oreg.x.dx;
   return(*x || *y);
}



/* Function 12. Set Interrupt Subroutine
                call mask and adddress    */
void Event_Set(unsigned int event_mask,
               void far *handler)
{
  union REGS ireg;
  struct SREGS sreg;

  ireg.x.ax = 12;
  ireg.x.cx = event_mask;
  ireg.x.dx = FP_OFF(handler);
  sreg.es   = FP_SEG(handler);
  int86x(0x33, &ireg, &ireg, &sreg);
}



/* Function 13. Light Pen emulation mode on  */
void Lightpen_Emu_On()
{
   union REGS ireg;

   ireg.x.ax = 13;
   int86(0x33, &ireg, &ireg);
}



/* Function 14. Light Pen emulation mode off  */
void Lightpen_Emu_Off()
{
   union REGS ireg;

   ireg.x.ax = 14;
   int86(0x33, &ireg, &ireg);
}



/* Function 15. Set Mickey/Pixel ratio  */
void Set_M2P_Ratio(int X_Ratio, int Y_Ratio)
{
   union REGS ireg;

   ireg.x.ax = 15;
   ireg.x.cx = X_Ratio;
   ireg.x.dx = Y_Ratio;
   int86(0x33, &ireg, &ireg);
}



/* Function 16. Set block of cursor hiding  */
void Block_Of_Hide(int Left, int Top,
                   int Right, int Bottom)
{
   union REGS ireg;

   ireg.x.ax = 16;
   ireg.x.cx = Left;
   ireg.x.dx = Top;
   ireg.x.si = Right;
   ireg.x.di = Bottom;
   int86(0x33, &ireg, &ireg);
}



/* Function 19. Set double speed threshold  */
void Double_Speed(int speed)
{
   union REGS ireg;

   ireg.x.ax = 19;
   ireg.x.dx = speed;
   int86(0x33, &ireg, &ireg);
}



/* Function 20. Swap interrupt subroutine   */
void far *Swap_Event_Set(unsigned int *event_mask,
                         void far *handler)
{
  union REGS ireg, oreg;
  struct SREGS sreg;
  void far *oldhandler;

  ireg.x.ax = 20;
  ireg.x.cx = *event_mask;
  ireg.x.dx = FP_OFF(handler);
  sreg.es   = FP_SEG(handler);
  int86x(0x33, &ireg, &oreg, &sreg);
  oldhandler = MK_FP(sreg.es, oreg.x.dx);
  *event_mask = oreg.x.cx;
  return(oldhandler);
}



/* Function 21.  Get the requirement of memory
                 for storing mouse status.      */
int Get_Storage_Req()
{
   union REGS ireg, oreg;

   ireg.x.ax = 21;
   int86(0x33, &ireg, &oreg);
   return(oreg.x.bx);
}



/* Function 22.  Save the mouse state.   */
void Save_Mouse_State(void *buf)
{
   union REGS ireg;
   struct SREGS isreg;

   ireg.x.ax = 22;
   ireg.x.dx = FP_OFF(buf);
   segread(&isreg);
   isreg.es = isreg.ds;
   int86x(0x33, &ireg, &ireg, &isreg);
}



/* Function 23.  Restore the mouse state.  */
void Restore_Mouse_State(void *buf)
{
   union REGS ireg;
   struct SREGS isreg;

   ireg.x.ax = 23;
   ireg.x.dx = FP_OFF(buf);
   segread(&isreg);
   isreg.es = isreg.ds;
   int86x(0x33, &ireg, &ireg, &isreg);
}

